@@ -1,7 +1,7 @@
 /*
   dummy cacheflush function (does nothing)
 
-  Copyright (C) 2007 Free Software Foundation, Inc.
+  Copyright (C) 2007,2011 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -24,6 +24,9 @@
    ./configure arm_cacheflush=arch/arm/cacheflush
  */
 #include <stdlib.h>
-void cacheflush(void *p, size_t size)
+void gforth_cacheflush(void *p, size_t size)
 {
+#ifdef __ANDROID_API__
+  cacheflush(p, p+size, 0);
+#endif
 }
