@@ -160,6 +160,29 @@ typedef struct {
 #define FLOORED_DIV ((1%-3)>0)
 #endif
 
+// prior to 4.8, gcc did not provide __builtin_bswap16 on some platforms so we emulate it
+// see http://gcc.gnu.org/bugzilla/show_bug.cgi?id=52624
+// Clang has a similar problem, but their feature test macros make it easier to detect
+#ifdef HAVE___BUILTIN_BSWAP16
+# define BSWAP16(x) __builtin_bswap16(x)
+#else
+# ifdef HAVE___BUILTIN_BSWAP32
+#  define BSWAP16(x) __builtin_bswap32((x) << 16)
+# else
+#  define BSWAP16(x) ((((x)>>8) & 0xff) | ((x)<<8) & 0xff00)
+# endif
+#endif
+#ifdef HAVE___BUILTIN_BSWAP32
+# define BSWAP32(x) __builtin_bswap32(x)
+#else
+# define BSWAP32(x) ((BSWAP16(x)<<16) | (BSWAP16((x)>>16)))
+#endif
+#ifdef HAVE___BUILTIN_BSWAP64
+# define BSWAP64(x) __builtin_bswap64(x)
+#else
+# define BSWAP64(x) ((BSWAP32(x)<<32) | (BSWAP32((x)>>32)))
+#endif
+
 #if defined(BUGGY_LONG_LONG)
 
 #define BUGGY_LL_CMP    /* compares not possible */
@@ -170,6 +193,7 @@ typedef struct {
 #define BUGGY_LL_D2F    /* to float not possible */
 #define BUGGY_LL_F2D    /* from float not possible */
 #define BUGGY_LL_SIZE   /* long long "too short", so we use something else */
+#define BUGGY_LL_SWAP   /* byteswap not possible */
 
 typedef struct {
   Cell hi;
@@ -380,21 +404,12 @@ struct F83Name {
 
 #define F83NAME_COUNT(np)	((np)->countetc & 0x1f)
 #endif
-struct Longname {
-  struct Longname *next;  /* the link field for old hands */
-  Cell		countetc;
-  char		name[0];
-};
 
-#ifdef OLD_HEADER
-# define LONGNAME_COUNT(np)	((np)->countetc & (((~((UCell)0))<<4)>>4))
-# define LONGNAME_NAME(np)      ((Char *)((np)->name))
-# define LONGNAME_NEXT(np)      ((np)->next)
-#else
-# define LONGNAME_COUNT(np)     ((((Cell*)np)[-3]) & (((~((UCell)0))<<4)>>4))
-# define LONGNAME_NAME(np)      ((Char *)(np)-3*sizeof(Cell)-LONGNAME_COUNT(np))
-# define LONGNAME_NEXT(np)      ((struct Longname*)(((Cell*)np)[-2]))
-#endif
+#define LONGNAME_OFF 3
+#define RESERVED_BITS 8
+#define LONGNAME_COUNT(np)     ((((Cell*)np)[-LONGNAME_OFF]) & (((~((UCell)0))<<RESERVED_BITS)>>RESERVED_BITS))
+#define LONGNAME_NAME(np)      ((Char *)(np)-LONGNAME_OFF*sizeof(Cell)-LONGNAME_COUNT(np))
+#define LONGNAME_NEXT(np)      ((struct Longname*)(((Cell*)np)[-2]))
 
 struct Cellpair {
   Cell n1;
@@ -440,6 +455,7 @@ Label *gforth_engine(Xt *ip sr_proto);
 Label *gforth_engine2(Xt *ip sr_proto);
 Label *gforth_engine3(Xt *ip sr_proto);
 
+
 Cell gforth_main(int argc, char **argv, char **env);
 int gforth_args(int argc, char ** argv, char ** path, char ** imagename);
 ImageHeader* gforth_loader(char* imagename, char* path);
@@ -449,13 +465,14 @@ void gforth_free();
 Cell gforth_go(Xt* ip0);
 Cell gforth_boot(int argc, char** argv, char* path);
 void gforth_bootmessage();
-void gforth_setwinch();
 Cell gforth_start(int argc, char ** argv);
 Cell gforth_quit();
 Xt gforth_find(Char * name);
 Cell gforth_execute(Xt xt);
 void gforth_cleanup();
 void gforth_printmetrics();
+void gforth_setwinch();
+void gforth_setstacks(user_area * t);
 #if defined(DOUBLY_INDIRECT)
 Cell gforth_make_image(int debugflag);
 #endif
