@@ -615,8 +615,7 @@ static void *dict_alloc_read(FILE *file, Cell imagesize, Cell dictsize, Cell off
   read_image:
     rewind(file);  /* fseek(imagefile,0L,SEEK_SET); */
     debugp(stderr,"try fread(%p, 1, %lx, file); ", image, imagesize);
-    fread(image, 1, imagesize, file);
-    if(ferror(file)) {
+    if(imagesize!=fread(image, 1, imagesize, file) || ferror(file)) {
       debugp(stderr, "failed\n");
       return NULL;
     } else {
@@ -2084,7 +2083,9 @@ ImageHeader* gforth_loader(char* imagename, char* path)
   termstate = make_termstate();
 #endif /* !(defined(DOUBLY_INDIRECT) || defined(INDIRECT_THREADED)) */
 
-  fread((void *)&header,sizeof(ImageHeader),1,imagefile);
+  if(sizeof(ImageHeader)!=fread((void *)&header, 1, sizeof(ImageHeader), imagefile)) {
+    fprintf(stderr, "ImageHeader read failed\n");
+  }
 
   set_stack_sizes(&header);
   
@@ -2110,7 +2111,9 @@ ImageHeader* gforth_loader(char* imagename, char* path)
     Cell reloc_size=((header.image_size-1)/sizeof(Cell))/8+1;
     Char reloc_bits[reloc_size];
     fseek(imagefile, preamblesize+header.image_size, SEEK_SET);
-    fread(reloc_bits, 1, reloc_size, imagefile);
+    if(reloc_size != fread(reloc_bits, 1, reloc_size, imagefile)) {
+      fprintf(stderr, "Image reloc bits read terminated early\n");
+    }
     gforth_relocate((Cell *)imp, reloc_bits, header.image_size, (Cell)header.base, vm_prims);
 #if 0
     { /* let's see what the relocator did */
@@ -2287,7 +2290,7 @@ int gforth_args(int argc, char ** argv, char ** path, char ** imagename)
 
   opterr=0;
   while (1) {
-    int option_index=0;
+    int option_index=0, oldoptind=optind;
     static struct option opts[] = {
       {"appl-image", required_argument, NULL, 'a'},
       {"image-file", required_argument, NULL, 'i'},
@@ -2337,7 +2340,7 @@ int gforth_args(int argc, char ** argv, char ** path, char ** imagename)
     
     switch (c) {
     case EOF: return 0;
-    case '?': optind--; return 0;
+    case '?': optind=oldoptind; return 0;
     case 'a': *imagename = optarg; return 0;
     case 'i': *imagename = optarg; break;
     case 'm': if((dictsize = convsize(optarg,sizeof(Cell)))==-1L) return 1; break;
@@ -2405,7 +2408,7 @@ Engine Options:\n\
 SIZE arguments consist of an integer followed by a unit. The unit can be\n\
   `b' (byte), `e' (element; default), `k' (KB), `M' (MB), `G' (GB) or `T' (TB).\n",
 	      argv[0]);
-      optind--;
+      optind=oldoptind;
       return 0;
     }
   }
@@ -2438,6 +2441,7 @@ Cell const * gforth_pointers(Cell n)
   case 6: return (Cell *)&gforth_main_UP;
   case 7: return (Cell *)&gforth_go;
   case 8: return (Cell *)&gforth_sigset;
+  case 9: return (Cell *)&gforth_setstacks;
   default: return NULL;
   }
 }
@@ -2511,15 +2515,24 @@ user_area* gforth_stacks(Cell dsize, Cell fsize, Cell rsize, Cell lsize)
     page_noaccess((void*)a); a+=pagesize; up0->fp0=(Float*)(a+fsize); a+=fsizep;
     page_noaccess((void*)a); a+=pagesize; up0->lp0=(Address)(a+lsize); a+=lsizep;
     page_noaccess((void*)a); a+=pagesize;
-#ifdef SIGSTKSZ
+# ifdef SIGSTKSZ
     sigstack.ss_sp=(void*)a+SIGSTKSZ;
     sigstack.ss_size=SIGSTKSZ;
     sigstack.ss_flags=0;
     sas_retval=sigaltstack(&sigstack,(stack_t *)0);
-#if defined(HAS_FILE) || !defined(STANDALONE)
+# if defined(HAS_FILE) || !defined(STANDALONE)
     if(sas_retval)
       debugp(stderr,"sigaltstack: %s\n",strerror(errno));
+# endif
 #endif
+  /* ensure that the cached elements (if any) are accessible */
+#if !(defined(GFORTH_DEBUGGING) || defined(INDIRECT_THREADED) || defined(DOUBLY_INDIRECT) || defined(VM_PROFILING))
+    up0->sp0 -= 8; /* make stuff below bottom accessible for stack caching */
+    up0->fp0--;
+#else
+# ifdef DEBUG
+    up0->sp0--; // debug will print TOS even if the stack is empty
+# endif
 #endif
     return up0;
   }
@@ -2571,25 +2584,15 @@ void gforth_free_stacks(user_area * t)
     debugp(stderr,"sucess\n");
 }
 
-void gforth_setstacks()
+void gforth_setstacks(user_area * t)
 {
   gforth_magic = GFORTH_MAGIC; /* mark task as maintained */
-  gforth_UP->next_task = 0; /* mark user area as need-to-be-set */
+  t->next_task = 0; /* mark user area as need-to-be-set */
 
-  /* ensure that the cached elements (if any) are accessible */
-#if !(defined(GFORTH_DEBUGGING) || defined(INDIRECT_THREADED) || defined(DOUBLY_INDIRECT) || defined(VM_PROFILING))
-  gforth_UP->sp0 -= 8; /* make stuff below bottom accessible for stack caching */
-  gforth_UP->fp0--;
-#else
-#ifdef DEBUG
-  gforth_UP->sp0--; // debug will print TOS even if the stack is empty
-#endif
-#endif
-
-  gforth_SP = gforth_UP->sp0;
-  gforth_RP = gforth_UP->rp0;
-  gforth_FP = gforth_UP->fp0;
-  gforth_LP = gforth_UP->lp0;
+  gforth_SP = t->sp0;
+  gforth_RP = t->rp0;
+  gforth_FP = t->fp0;
+  gforth_LP = t->lp0;
 }
 
 Cell gforth_boot(int argc, char** argv, char* path)
@@ -2676,7 +2679,7 @@ Cell gforth_start(int argc, char ** argv)
   if(gforth_header==NULL)
     return -59; /* allocate error */
   gforth_main_UP = gforth_UP = gforth_stacks(dsize, fsize, rsize, lsize);
-  gforth_setstacks();
+  gforth_setstacks(gforth_UP);
   return gforth_boot(argc, argv, path);
 }
 
