@@ -1,6 +1,6 @@
 /* a strerror implemenation
 
-  Copyright (C) 1995,2000,2003,2007 Free Software Foundation, Inc.
+  Copyright (C) 1995,2000,2003,2007,2014,2015 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -21,8 +21,8 @@
 #include <errno.h>
 #include <stdio.h>
 
-extern char *sys_errlist[];
-extern int sys_nerr;
+extern const char *const sys_errlist[];
+extern const int sys_nerr;
 
 char *strerror(int err)
 {
