@@ -44,7 +44,7 @@
 extern int debug;
 # define debugp(x...) do { if (debug) fprintf(x); } while (0)
 #endif
-#ifdef HAVE_MCHECK
+#if defined(HAVE_MCHECK)
 pthread_mutex_t memlock = PTHREAD_MUTEX_INITIALIZER;
 void* (*malloc_l)(size_t size)=malloc;
 void (*free_l)(void* addr)=free;
@@ -963,7 +963,7 @@ DCell smdiv (DCell num, Cell denom)
   vm_twoCell2d(q,r,res);
 #else
   UDCell ures;
-  UCell l, q, r;
+  UCell MAYBE_UNUSED l, q, r;
   Cell h;
   Cell denomsign=denom;
 
